<?php
/**
Functions for Ship-Quik
 **/

namespace ShipQuik\Shared;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ConnectException;
USE GuzzleHttp\Exception\RequestException;

class ShipQuik_API
{
    private static $_instance = null;

    protected $_params = [];

    protected $parent = null;

    protected $config = null;

    protected $_is_post = true;

    public function __construct($parent = null, $params = [], $is_post = true)
    {
        $this->parent  = $parent;
        $this->_params  = $params;
        $this->_is_post = $is_post;

        $this->config   = $parent->config;
    }

    public function runCall($connecttimeout = 800, $timeout = 800)
    {
        try {
            $client = new Client();

            $headers = [
                'Content-Type' => 'application/json; charset=utf-8'
            ];

            $token = $this->config->getAccessToken();

            if ($token) {
                $headers['access-token'] = $token;
            }

            try {
                if ($this->_is_post) {
                    $response = $client->request('POST', $this->_params['Baseurl'], [
                        'headers' => $headers,
                        'body' => json_encode($this->_params['Data']),
                    ]);
                } else {
                    $response = $client->request('GET', $this->_params['Baseurl'], [
                        'headers' => $headers,
                        'query' => $this->_params['Data']
                    ]);
                }
            } catch (RequestException $e) {
                if ($e->getCode() == 401) {
                    return "renew_token";
                }
                $response = $e->getResponse();
                if ($response) {
                    $message = json_decode($response->getBody()->getContents(), true);
                    if ($message) {
                        $this->parent->logger->info(
                            'CURL: Response: ' . wc_print_r($message, true),
                            array('source' => 'ship-quik')
                        );
                    }
                } else {
                    $message = $e->getMessage();
                    if ($message) {
                        $this->parent->logger->info(
                            'CURL: Response: ' . wc_print_r($message, true),
                            array('source' => 'ship-quik')
                        );
                    }
                }

                return false;
            }

            $data = json_decode($response->getBody()->getContents(), true);

            return $data;

        } catch (ConnectException $e) {
            $this->parent->logger->info(
                'CURL: Error: ' . wc_print_r($e->getMessage(), true),
                array('source' => 'ship-quik')
            );
        }

        unset($response);
        unset($client);

        return false;
    }

    public static function instance()
    {
        if (! isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }
        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
