<?php
/**
Functions for Ship-Quik
 **/

namespace ShipQuik\Shared;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use ShipQuik\WooCommerce\ShipQuik_Shipments;

class ShipQuik_ShippingPickupInfo
{
    private static $_instance = null;

    protected $parent;

    protected $pickupInfo;

    protected $config;

    protected $data;

    public function __construct($parent = null, $pickupInfo = [])
    {
        $this->parent = $parent;
        $this->pickupInfo = $pickupInfo;

        $this->config = $parent->config;

        $this->data = $this->getData($this->pickupInfo);
    }

    private function getData($pickupInfo)
    {
        $pickupInfoData = [];

        $customerAddress = $pickupInfo['destination'];

        $senderAddress = ShipQuik_Shipments::getSenderAddress($this->config);

        $pickupInfoData['supplier'] = $pickupInfo['supplier'];
        $pickupInfoData['service'] = $pickupInfo['service'];

        $pickupInfoData['origin']['countryCode'] = $senderAddress['CountryCode'];
        $pickupInfoData['origin']['postalCode'] = $senderAddress['PostalCode'];
        $pickupInfoData['destination']['countryCode'] = $customerAddress['country'];
        $pickupInfoData['destination']['postalCode'] = $customerAddress['postcode'];

        $pickupInfoData['isKG'] = true;

        $pickupInfoData['parcels'] = $pickupInfo['boxes'];

        return $pickupInfoData;
    }

    public function send()
    {
        $result = false;

        $params = [];

        $params['Baseurl'] = $this->config->getValueByKey('endpoints')['getSuppliersPickupInfo'];
        $params['Data'] = $this->data;

        // d(json_encode($params['Data']));

        $curl = new ShipQuik_API($this->parent, $params, true);
        $result = $curl->runCall();

        if ($result == 'renew_token') {
            $this->config->reNewAccessToken();

            $result = $curl->runCall();
        }

        if ($result) {
            if (isset($result['data'])) {
                return $result;
            }
        }

        return $result;
    }

    public static function instance()
    {
        if (!isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }

        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
