<?php
/**
Functions for Ship-Quik
 **/

namespace ShipQuik\Shared;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use \ShipQuik\Shared\ShipQuik_API;
use \ShipQuik\WooCommerce\ShipQuik_Shipments;

use DateTime;

class ShipQuik_Supplier
{
    private static $_instance = null;

    protected $parent;

    protected $boxes;

    protected $destination;

    protected $config;

    protected $data;

    public function __construct($parent = null, $boxes = [], $destination = [])
    {
        $this->parent = $parent;
        $this->boxes = $boxes;
        $this->destination = $destination;

        $this->config = $parent->config;

        $this->data = $this->getData($boxes, $destination);
    }

    private function getNormalizeDestination($destination)
    {
        $resultAddress = [];

        // package destination
        if (isset($destination['country'])) {
            $resultAddress['CountryCode'] = $destination['country'];
            $resultAddress['Country']= WC()->countries->countries[$destination['country']];
        }
        if (isset($destination['postcode'])) {
            $resultAddress['PostalCode'] = $destination['postcode'];
        }
        if (isset($destination['city'])) {
            $resultAddress['City'] = $destination['city'];
        }
        if (isset($destination['address_1'])) {
            $resultAddress['Street'] = $destination['address_1'] . ' ' . $destination['address_2'];
        }

        // customer destination
        if (isset($destination['countryCode'])) {
            $resultAddress['CountryCode'] = $destination['countryCode'];
            $resultAddress['Country']= WC()->countries->countries[$destination['countryCode']];
        }
        if (isset($destination['postalCode'])) {
            $resultAddress['PostalCode'] = $destination['postalCode'];
        }
        if (isset($destination['street'])) {
            $resultAddress['Street'] = $destination['street'];
        }

        return $resultAddress;
    }

    // HACK: PS
    // public static function getNormalizeDestinationPrestaShop($destination)
    // {
    //     $resultAddress = [];

    //     // package destination
    //     $country = new \Country($destination->id_country);
    //     $resultAddress['countrycode'] = $country->iso_code;

    //     if (isset($destination->country)) {
    //         $resultAddress['country'] = $destination->country;
    //     }
    //     if (isset($destination->postcode)) {
    //         $resultAddress['postcode'] = $destination->postcode;
    //     }
    //     if (isset($destination->city)) {
    //         $resultAddress['city'] = $destination->city;
    //     }
    //     if (isset($destination->address1)) {
    //         $resultAddress['street'] = $destination->address1 . ' ' . $destination->address2;
    //     }
    //     if (isset($destination->phone)) {
    //         $resultAddress['phone'] = $destination->phone;
    //     }
    //     if (isset($destination->company)) {
    //         $resultAddress['company'] = $destination->company;
    //     }
    //     if (isset($destination->firstname) && isset($destination->lastname)) {
    //         $resultAddress['contact'] = $destination->firstname . ' ' . $destination->firstname;
    //     }

    //     $state = new \State($destination->id_state);
    //     $resultAddress['state'] = $state->name;

    //     return $resultAddress;
    // }

    private function getData($boxes, $destination)
    {
        $supplierData = [];

        $customerAddress = $this->getNormalizeDestination($destination);
        $senderAddress = ShipQuik_Shipments::getSenderAddress($this->config);

        $supplierData['onlyPrice']              = true;

        $supplierData['countryOrigin']          = $senderAddress['Country'];
        $supplierData['countryDestination']     = $customerAddress['Country'];
        $supplierData['postalCodeOrigin']       = $senderAddress['PostalCode'];
        $supplierData['postalCodeDestination']  = $customerAddress['PostalCode'];
        $supplierData['countryCodeOrigin']      = $senderAddress['CountryCode'];
        $supplierData['countryCodeDestination'] = $customerAddress['CountryCode'];
        $supplierData['cityOrigin']             = $senderAddress['City'];
        $supplierData['cityDestination']        = $customerAddress['City'];
        $supplierData['addressDescOrigin']      = $senderAddress['Street'];
        $supplierData['addressDescDestination'] = $customerAddress['Street'];

        $supplierData['pesoEnKG']               = true;

        $supplierData['boxes']                  = $boxes;

        $supplierData['uID']                    = (string) $this->config->getIdCliente();

        return $supplierData;
    }

    public function send($supplier, $pickupInfo)
    {
        $result = false;

        $params = [];

        $params['Baseurl']            = $this->config->getValueByKey('endpoints')['getSupplierBaseUri'];
        $params['Data']               = $this->data;
        $params['Data']['suppliers'][]  = $supplier;
        $params['Data']['searchMode'] =  "none";
        $params['Data']['pickupInfo'] =  $pickupInfo;
        $params['Data']['onlyPrice'] =  true;

        // d(json_encode($params['Data']));

        $curl = new ShipQuik_API($this->parent, $params, true);
        $result = $curl->runCall();

        if ($result == "renew_token") {
            $this->config->reNewAccessToken();

            $result = $curl->runCall();
        }

        $resultArray = [];

        if ($result) {
            foreach ($result as $element) {
                foreach ($element as $key => $services) {
                    foreach ($services as $service) {
                        unset($service['serviceObservations']);
                        $resultArray[] = $service;
                    }
                }
            }
        }

        return $resultArray;
    }

    public function getSuppliersResult($suppliers = [])
    {
        if (empty($suppliers)) {
            $suppliers = $this->config->getValueByKey('suppliers');
        }

        $suppliersResult = $this->send(unserialize($suppliers));

        return $suppliersResult;
    }

    public function getServiceByName($shipQuikSuppliers, $name)
    {
        foreach ($shipQuikSuppliers as $service) {
            if ($service['service'] == $name) {
                return $service;
            }
        }

        return false;
    }

    public static function instance()
    {
        if (! isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }
        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
