<?php
/**
Functions for Ship-Quik
 **/

namespace ShipQuik\Shared;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use \ShipQuik\Shared\ShipQuik_API;
use \ShipQuik\WooCommerce\ShipQuik_Shipments;

use DateTime;

class ShipQuik_Suppliers
{
    private static $_instance = null;

    protected $parent;

    protected $boxes;

    protected $destination;

    protected $id_carrier;

    protected $config;

    protected $data;

    public function __construct($parent = null, $boxes = [], $destination = [], $id_carrier = 0)
    {
        $this->parent = $parent;
        $this->boxes = $boxes;
        $this->destination = $destination;

        $this->config = $parent->config;

        $this->data = $this->getData($this->boxes, $this->destination);
        $this->id_carrier = $id_carrier;
    }

    private function getNormalizeDestination($destination)
    {
        $resultAddress = [];

        // package destination
        if (isset($destination['country'])) {
            $resultAddress['CountryCode'] = $destination['country'];
            $resultAddress['Country']= WC()->countries->countries[$destination['country']];
        }
        if (isset($destination['postcode'])) {
            $resultAddress['PostalCode'] = $destination['postcode'];
        }
        if (isset($destination['city'])) {
            $resultAddress['City'] = $destination['city'];
        }
        if (isset($destination['address'])) {
            $resultAddress['Street'] = $destination['address'];
        }

        // customer destination
        if (isset($destination['countryCode'])) {
            $resultAddress['CountryCode'] = $destination['countryCode'];
            $resultAddress['Country']= WC()->countries->countries[$destination['countryCode']];
        }
        if (isset($destination['postalCode'])) {
            $resultAddress['PostalCode'] = $destination['postalCode'];
        }
        if (isset($destination['street'])) {
            $resultAddress['Street'] = $destination['street'];
        }

        return $resultAddress;
    }

    private function getData($boxes, $destination)
    {
        $supplierData = [];

        $customerAddress = $this->getNormalizeDestination($destination);
        $senderAddress = ShipQuik_Shipments::getSenderAddress($this->config);

        $supplierData['onlyPrice']              = true;

        $supplierData['postalCodeOrigin']       = $senderAddress['PostalCode'];
        $supplierData['postalCodeDestination']  = $customerAddress['PostalCode'];
        $supplierData['countryCodeOrigin']      = $senderAddress['CountryCode'];
        $supplierData['countryCodeDestination'] = $customerAddress['CountryCode'];

        $supplierData['pesoEnKG']               = true;

        $supplierData['boxes']                  = $boxes;

        $supplierData['uID']                    = (string) $this->config->getIdCliente();

        return $supplierData;
    }

    public function send($suppliers, $searchMode)
    {
        $result = false;

        $params = [];

        if ($searchMode === 'speed') {
            $searchMode = 'lowest-speed';
        }
        if ($searchMode === 'price') {
            $searchMode = 'lowest-price';
        }
        if ($searchMode === 'speedprice') {
            $searchMode = 'lowest-both';
        }

        $params['Baseurl']            = $this->config->getValueByKey('endpoints')['getSupplierBaseUri'];
        $params['Data']               = $this->data;
        $params['Data']['suppliers']  = $suppliers;
        $params['Data']['searchMode'] = $searchMode;
        $params['Data']['pickupInfo'] = true;

        $curl = new ShipQuik_API($this->parent, $params, true);
        $result = $curl->runCall();

        if ($result == "renew_token") {
            $this->config->reNewAccessToken();
            $result = $curl->runCall();
        }

        $resultArray = [];

        if ($result) {
            foreach ($result as $element) {
                unset($element['serviceObservations']);
                $resultArray[] = $element;
            }
        }

        return $resultArray;
    }

    public function getSuppliersResult($suppliers = [], $pricingMode)
    {
        if (empty($suppliers)) {
            $suppliers = $this->config->getValueByKey('suppliers');
        }

        $suppliersResult = $this->send($suppliers, $pricingMode); // HACK: PS unserialize

        return $suppliersResult;
    }

    public function getServiceByName($shipQuikSuppliers, $name)
    {
        foreach ($shipQuikSuppliers as $service) {
            if ($service['service'] == $name) {
                return $service;
            }
        }

        return false;
    }

    public function getServiceByType($shipQuikSuppliers, $type)
    {
        foreach ($shipQuikSuppliers as $service) {
            if ($service['type'] == $type) {
                return $service;
            }
        }

        return false;
    }

    public static function instance()
    {
        if (! isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }
        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
