<?php
/**
Functions for Ship-Quik
 **/

namespace ShipQuik\WooCommerce\Admin;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use \ShipQuik\WooCommerce\ShipQuik_ShippingRate;
use \ShipQuik\WooCommerce\ShipQuik_Shipments;
use \ShipQuik\WooCommerce\FrontEnd\ShipQuik_CheckOut;
use \ShipQuik\WooCommerce\ShipQuik_ShippingPackage;

use \ShipQuik\Shared\ShipQuik_Helper;
use \ShipQuik\Shared\ShipQuik_Configuration;
use \ShipQuik\Shared\ShipQuik_PickupDate;
use \ShipQuik\Shared\ShipQuik_OrdersPaid;
use \ShipQuik\Shared\ShipQuik_Supplier;

use GlobIterator;

class ShipQuik_OrdersAdmin
{
    /**
     * Parent object.
     *
     * @var    object
     * @access private
     * @since  1.0.0
     */
    protected $parent = null; 

    /**
     * General configuration.
     *
     * @var    object
     * @access public
     * @since  1.0.0
     */
    protected $config = null;

    public function __construct($parent)
    {
        $this->parent = $parent;
        $this->config = $parent->config;
    }

    public function shop_order_column($columns)
    {
        $reordered_columns = array();

        foreach ($columns as $key => $column) {
            $reordered_columns[$key] = $column;

            if ($key == 'order_status') {
                $reordered_columns['ship_quik_pickup'] = __('Ship-Quik Pickup Date/Time', 'ship-quik');
                $reordered_columns['ship_quik_cost'] = __('Ship-Quik Cost', 'ship-quik');
                $reordered_columns['ship_quik_status'] = __('Ship-Quik Payment', 'ship-quik');
                $reordered_columns['ship_quik_action'] = __('Ship-Quik Action', 'ship-quik');
            }
        }

        return $reordered_columns;
    }

    public function hidden_order_itemmeta( $array ) {
        $array[] = '_supplier_data';
        $array[] = '_logo';

        return $array; 
    }

    public function payment_return() {
        $returnUrl = esc_url(admin_url('edit.php?post_type=shop_order'));

        // 1. Redirect to payment form
        if (isset($_REQUEST['paymentKey'])) {

            $paymentKey = sanitize_text_field($_REQUEST['paymentKey']);
            $ordersId = sanitize_text_field($_REQUEST['ordersId']);

            if ($paymentKey != 'nopaymentkey' && $ordersId > 0) {
                $returnSubmitUrl = esc_url($returnUrl . '&ordersId=' . sanitize_text_field($_REQUEST['ordersId']));

                $paymentTotal = sanitize_text_field($_REQUEST['paymentTotal']);

                echo '<meta name="paymentKey" content="' . esc_attr($paymentKey) . '"></meta>';
                echo '<meta name="ordersId" content="' . esc_attr($ordersId) . '"></meta>';
                echo '<meta name="paymentTotal" content="' . esc_attr($paymentTotal) . '"></meta>';
                echo '<meta name="paymentSubmitReturnUrl" content="' . esc_url($returnSubmitUrl) . '"></meta>';
                echo '<meta name="paymentCancelReturnUrl" content="' . esc_url($returnUrl) . '"></meta>';
            }
        }

        // 2. Redirect from payment form
        if (isset($_REQUEST['payment_return'])
            && isset($_REQUEST['payment_intent'])
            && isset($_REQUEST['payment_intent_client_secret'])
            && isset($_REQUEST['redirect_status'])
            && isset($_REQUEST['ordersId'])) {

            $payment_return = sanitize_text_field($_REQUEST['payment_return']);
            $payment_intent = sanitize_text_field($_REQUEST['payment_intent']);
            $payment_intent_client_secret = sanitize_text_field($_REQUEST['payment_intent_client_secret']);
            $redirect_status = sanitize_text_field($_REQUEST['redirect_status']);
            $ordersId = sanitize_text_field($_REQUEST['ordersId']);

            $pending = explode(',', $ordersId);
            $ordersPending = $this->haveOrdersPending($pending);

            if ($redirect_status == 'succeeded') {
                if ($ordersPending['condition']) {
                    foreach ($ordersPending['values'] as $order_id) {
                        $order = wc_get_order($order_id);
                        $order->update_meta_data('_shipquik_payment_status', $payment_intent);
                        $order->save_meta_data();
                    }
                    wp_redirect($returnUrl . '&processShipQuikOrdersModal=1&ordersId=' . $ordersId);
                    exit;
                }
            } else {
                // Error en el pago.
                add_action('admin_notices', [ $this, 'order_payment_bulk_action_admin_notice' ]);
                wp_redirect($returnUrl);
                exit;
            }
        }

        // 3. Process orders
        if (isset($_REQUEST['processShipQuikOrdersModal'])) {
            echo '<meta name="processShipQuikOrdersModal" content="1"></meta>';
            $ordersId = sanitize_text_field($_REQUEST['ordersId']);
            echo '<meta name="ordersId" content="' . esc_attr($ordersId) . '"></meta>';
            echo '<meta name="returnUrl" content="' . esc_url($returnUrl) . '"></meta>';
        }

        // 3. Printing orders
        if (isset($_REQUEST['print_labels'])) {
            echo '<meta name="print_labels" content="1"></meta>';
            $ordersId = sanitize_text_field($_REQUEST['ordersId']);
            echo '<meta name="ordersId" content="' . esc_attr($ordersId) . '"></meta>';
            echo '<meta name="returnUrl" content="' . esc_url($returnUrl) . '"></meta>';
            echo '<meta name="printProducts" content="' . esc_attr($_REQUEST['printProducts']) . '"></meta>';
        }
    }

    private function haveOrdersPending($orders)
    {
        $totalPayment = 0;
        $ordersId = array();

        foreach ($orders as $order_id) {
            $order = wc_get_order($order_id);

            $payment_status = $order->get_meta('_shipquik_payment_status');
            $payment        = $order->get_meta('_shipquik_payment');

            if (isset($payment->data)) {
                $payment = $payment->data;
            }

            $trackingNumber  = $order->get_meta('_shipquik_tracking_number');
            $pickupNumber   = $order->get_meta('_shipquik_pickup_number');
            $fileType       = $order->get_meta('_shipquik_label_type');

            if (!$trackingNumber && !$pickupNumber && !$fileType && $order->get_meta('_shipquik_payment_status') == '0') {
                $totalPayment = $totalPayment + $payment['payment']['total'];
                $ordersId[] = $order_id;
            }
        }

        return [ 'condition' => ($totalPayment > 0 ? true : false), 'values' => $ordersId ];
    }

    public function order_bulk_actions($actions)
    {
        if ($this->config->getPaymentMethod() == 1) {
            $actions['order_payment'] = __('Ship-Quik Order Payment', 'ship-quik');
        } else {
            $actions['orders_processing'] = __('Ship-Quik Orders Processing', 'ship-quik');
            $actions['orders_checking'] = __('Ship-Quik Check Orders', 'ship-quik');
        }
        $actions['order_printing'] = __('Ship-Quik Printing Orders', 'ship-quik');

        return $actions;
    }

    function order_payment_bulk_action_edit_shop_order($redirect_to, $action, $orders_ids)
    {
        if ($action !== 'order_payment' && $action !== 'orders_processing' && $action !== 'order_printing' && $action !== 'orders_checking')
            return $redirect_to;

        if ($action === 'order_payment') {
            $ordersId = array();
            $references = array();
            $orders_data = array();
            $totalPayment   = 0;

            foreach ($orders_ids as $order_id) {
                $order = wc_get_order($order_id);

                $label_error    = $order->get_meta('_shipquik_label_error');
                $payment_status = $order->get_meta('_shipquik_payment_status');
                $payment        = $order->get_meta('_shipquik_payment');

                if (isset($payment->data)) {
                    $payment = $payment->data;
                }

                $trackingNumber  = $order->get_meta('_shipquik_tracking_number');
                $pickupNumber   = $order->get_meta('_shipquik_pickup_number');
                $fileType       = $order->get_meta('_shipquik_label_type');
                $pickupDatetime    = $order->get_meta('_shipquik_pickupdatetime');

                if (!$trackingNumber
                    && !$pickupNumber
                    && !$fileType
                    && $payment_status == "0"
                    && $payment
                    && !$label_error
                    && $pickupDatetime) {

                    $totalPayment = $totalPayment + $payment['payment']['total'];
                    $ordersId[] = $order_id;
                    $references[] = $payment['reference'];

                    if (ShipQuik_Configuration::get_option('woocommerce_ship_to_destination') == 'shipping') {
                        $customerAddress = $order->get_address('shipping');
                    } else {
                        $customerAddress = $order->get_address('billing');
                    }

                    $senderAddress = ShipQuik_Shipments::getSenderAddress($this->config);
                    $shippingMethod = [];

                    $shippingMethods = $order->get_shipping_methods();

                    // TODO: We must have only one.
                    foreach ($shippingMethods as $key => $value) {
                        $shippingMethod = $value;
                    }

                    foreach ($shippingMethod->get_meta_data() as $meta) {
                        $item = $meta->get_data();
                        if ($item['key'] == '_supplier_data') {
                            $supplierInfo = json_decode(str_replace('&quot;', '"', $item['value']), true, 99);

                            break;
                        }
                    }

                    $orders_data[] = [
                        'Nro de Orden'       => $order_id,
                        'Nombre Recogida'    => $this->config->getValueByKey('senderAddressCompany'),
                        'Direccion Recogida' => $senderAddress['Street'],
                        'Nombre Entrega'     => $customerAddress['first_name'] . ' ' . $customerAddress['last_name'],
                        'Direccion Entrega'  => $customerAddress['address_1'] . ' ' . $customerAddress['address_2'],
                        'Email Facturacion'  => $this->config->getValueByKey('senderAddressEmail'),
                        'Proveedor'          => $supplierInfo['supplier'],
                        'Servicio'           => $supplierInfo['service'],
                    ];
                }
            }

            $requestParams = [];

            if ($totalPayment > 0) {
                $senderAddress = ShipQuik_Shipments::getSenderAddress($this->config);

                $data = [
                    'currency'             => 'eur',
                    'amount'               => intval($totalPayment * 100),
                    'payment_method_types' => ['card'],
                    'metadata' => [
                        'Nro de Orden', json_encode($references)
                    ],
                    'setup_future_usage' => 'off_session',
                    'customer'    => [
                        'email' => $this->config->getValueByKey('activationEmail'),
                        'name'  => $this->config->getValueByKey('senderAddressCompany'),
                        'phone' => $this->config->getValueByKey('senderAddressPhoneNumber'),
                        'metadata' => [
                            'cif'     => $this->config->getValueByKey('senderAddressCif'),
                            'address' => $senderAddress['Street']
                        ]
                    ]
                ];

                $stripeIntent = ShipQuik_Helper::createStripeIntent($this->parent, $data);

                $requestParams = array(
                    'order_payment'   => '1',
                    'processed_count' => count($ordersId),
                    'ordersId'        => implode(',', $ordersId),
                    'paymentTotal'    => $totalPayment,
                    'paymentEmail'    => $this->config->getValueByKey('activationEmail'),
                    'paymentKey'      => (isset($stripeIntent['clientSecret']) ? $stripeIntent['clientSecret'] : 'nopaymentkey')
                );
            }

            remove_query_arg(array_keys($_REQUEST));

            return $redirect_to = add_query_arg($requestParams, $redirect_to);
        }

        if ($action === 'orders_processing') {
            $ordersId = array();

            foreach ($orders_ids as $order_id) {
                $order = wc_get_order($order_id);

                $label_error    = $order->get_meta('_shipquik_label_error');
                $trackingNumber = $order->get_meta('_shipquik_tracking_number');
                $pickupNumber   = $order->get_meta('_shipquik_pickup_number');
                $fileType       = $order->get_meta('_shipquik_label_type');
                $pickupDatetime = $order->get_meta('_shipquik_pickupdatetime');

                if (!$trackingNumber
                    && !$pickupNumber
                    && !$fileType
                    && !$label_error
                    && $pickupDatetime) {

                        $ordersId[] = $order_id;
                }
            }
            $requestParams = array(
                'processShipQuikOrdersModal' => '1',
                'ordersId' => implode(',', $ordersId),
            );

            return $redirect_to = add_query_arg($requestParams, $redirect_to);
        }

        if ($action === 'order_printing') {
            $ordersLabels = [];

            foreach ($orders_ids as $order_id) {
                $order = wc_get_order($order_id);

                $label_error    = $order->get_meta('_shipquik_label_error');
                $trackingNumber  = $order->get_meta('_shipquik_tracking_number');
                $pickupNumber   = $order->get_meta('_shipquik_pickup_number');

                if ($trackingNumber
                    && $pickupNumber
                    && !$label_error) {
                    $ordersLabels[] = $order_id;
                }
            }

            $requestParams = array(
                'returnUrl' => $redirect_to,
                'print_labels'   => '1',
                'ordersId'        => implode(',', $ordersLabels),
                'printProducts'    => $this->config->getValueByKey('printProducts'),
            );

            remove_query_arg(array_keys($_REQUEST));

            return $redirect_to = add_query_arg($requestParams, $redirect_to);
        }

        if ($action === 'orders_checking') {
            $clientId = $this->config->getIdCliente();
            $references = [];

            foreach ($orders_ids as $order_id) {
                $order = wc_get_order($order_id);

                $paymentStatus = $order->get_meta('_shipquik_payment_status');

                if ($paymentStatus == '0') {
                    $shipmentInfo = $order->get_meta('_shipquik_payment');

                    if (isset($shipmentInfo['reference'])) {
                        $references[] = [
                            "reference" => $shipmentInfo['reference'],
                            "orderId"   => $order_id,
                        ];
                    }
                }
            }

            $ordersPaid = new ShipQuik_OrdersPaid($this->parent, $clientId, $references);
            $result = $ordersPaid->send();

            if ($result) {
                foreach ($result['data'] as $key => $element) {
                    if ($element['paid']) {
                        $order = wc_get_order($references[$key]['orderId']);
                        $order->update_meta_data('_shipquik_payment_status', '1');

                        $order->save_meta_data();
                    }
                }
            }

            return $redirect_to;
        }
    }

    public function order_payment_bulk_action_admin_notice()
    {
        echo '<div id="message" class="updated fade"><p>' . __('There was a problem with your payment. Retry again.', 'ship-quik') . '</p></div>';
    }

    public function manage_shop_order_posts_custom_column($column, $order_id)
    {
        global $wp;

        $order = wc_get_order($order_id);
        $current_url = home_url(add_query_arg(array(), $wp->request . '?' . $wp->query_string));

        if ($order) {
            $shippingMethods = $order->get_shipping_methods();
    
            $shippingMethod = [];
    
            // TODO: We must have only one.
            foreach ($shippingMethods as $key => $value) {
                $shippingMethod = $value;
            }
    
            if (ShipQuik_ShippingRate::isShipQuik($shippingMethod)) {
                switch ($column) {
                    case 'ship_quik_pickup' :
                        $pickupDatetime = $order->get_meta('_shipquik_pickupdatetime');
                        $payment        = $order->get_meta('_shipquik_payment');

                        if (isset($payment->data)) {
                            $payment = $payment->data;
                        }

                        $payment_status = $order->get_meta('_shipquik_payment_status');
                        $trackingNumber  = $order->get_meta('_shipquik_tracking_number');
                        $pickupNumber   = $order->get_meta('_shipquik_pickup_number');
        
                        $minDate = date('Y-m-d');

                        if ($pickupDatetime) {
                            if ($payment_status == '0' && !$trackingNumber && !$pickupNumber) {
                                echo '<select class="ship-quik-picktime ship-quik-picktime_' . esc_attr($order_id) . '" data-key="' . esc_attr($order_id) . '" name="shipquik_pickuptime">';
                                    if ($pickupDatetime['pickup_time'] == '1') {
                                        echo '<option value="1" selected="selected">' . __('Mañana', 'ship-quik') . '</option>';
                                    } else {
                                        echo '<option value="1">' . __('Mañana', 'ship-quik') . '</option>';
                                    }
                                    if ($pickupDatetime['pickup_time'] == '2') {
                                        echo '<option value="2" selected="selected">' . __('Tarde', 'ship-quik') . '</option>';
                                    } else {
                                        echo '<option value="2">' . __('Tarde', 'ship-quik') . '</option>';
                                    }
                                echo '</select>';
                                echo '<input class="ship-quik-pickdate ship-quik-pickdate_' . esc_attr($order_id) . '" min="' . esc_attr($minDate) . '"  data-key="' . esc_attr($order_id) . '" type="date" format="YYYY/MM/DD" name="shipquik_pickupdate" value="' . esc_attr($pickupDatetime['pickup_date']) . '"/>';
                            } else {
                                echo '<select disabled="disabled" class="ship-quik-picktime ship-quik-picktime_' . esc_attr($order_id) . '" data-key="' . esc_attr($order_id) . '" name="shipquik_pickuptime">';
                                    if ($pickupDatetime['pickup_time'] == '1') {
                                        echo '<option value="1" selected="selected">' . __('Mañana', 'ship-quik') . '</option>';
                                    } else {
                                        echo '<option value="1">' . __('Mañana', 'ship-quik') . '</option>';
                                    }
                                    if ($pickupDatetime['pickup_time'] == '2') {
                                        echo '<option value="2" selected="selected">' . __('Tarde', 'ship-quik') . '</option>';
                                    } else {
                                        echo '<option value="2">' . __('Tarde', 'ship-quik') . '</option>';
                                    }
                                echo '</select>';
                                echo '<input disabled="disabled" class="ship-quik-pickdate ship-quik-pickdate_' . esc_attr($order_id) . '" min="' . esc_attr($minDate) . '"  data-key="' . esc_attr($order_id) . '" type="date" format="YYYY/MM/DD" name="shipquik_pickupdate" value="' . esc_attr($pickupDatetime['pickup_date']) . '"/>';
                            }
                        } else {
                            echo '<select class="ship-quik-picktime ship-quik-pickdate-red ship-quik-picktime_' . esc_attr($order_id) . '" data-key="' . esc_attr($order_id) . '" name="shipquik_pickuptime">';
                                echo '<option value="1">' . __('Mañana', 'ship-quik') . '</option>';
                                echo '<option value="2">' . __('Tarde', 'ship-quik') . '</option>';
                            echo '</select>';
                            echo '<input class="ship-quik-pickdate ship-quik-pickdate-red ship-quik-pickdate_' . esc_attr($order_id) . '" min="' . esc_attr($minDate) . '"  data-key="' . esc_attr($order_id) . '" type="date" format="YYYY/MM/DD" name="shipquik_pickupdate" value=""/>';
                        }

                        break;
                    case 'ship_quik_cost' :
                        $payment        = $order->get_meta('_shipquik_payment');

                        if (isset($payment->data)) {
                            $payment = $payment->data;
                        }

                        $payment_status = $order->get_meta('_shipquik_payment_status');

                        echo (isset($payment['payment']['total']) ? '<p><span><b>' . __('Import', 'ship-quik') . '</span>:&nbsp;' . wc_price($payment['payment']['total'], 2) . '</b></p>' : '');

                        break;
                    case 'ship_quik_status' :
                        $payment_status = $order->get_meta('_shipquik_payment_status');
                        $shipment        = $order->get_meta('_shipquik_payment');
                        if (isset($shipment->data)) {
                            $shipment = $shipment->data;
                        }

                        echo '<p>' . ($payment_status == '0' ? '<small style="color: red;">' . __('Unpaid', 'ship-quik') . '</small>' : '<small style="color: green;">' . __('Paid', 'ship-quik'))  . '</small>' . '</p>';

                        if (isset($shipment['reference'])) {
                            echo '<p>' . $shipment['reference'] . '</p>';
                        }
                        break;
                    case 'ship_quik_action' :
                        $payment_status = $order->get_meta('_shipquik_payment_status');

                        $trackingNumber  = $order->get_meta('_shipquik_tracking_number');
                        $pickupNumber   = $order->get_meta('_shipquik_pickup_number');
                        $fileType       = $order->get_meta('_shipquik_label_type');
                        $label_printed  = $order->get_meta('_shipquik_label_printed');
                        $label_error    = $order->get_meta('_shipquik_label_error');

                        if ($label_error == 'ERROR') {
                            echo '<span title="Error pedido" class="material-symbols-outlined ship-quik-error ship-quik-error_' . esc_attr($order_id) . '" data-key="' . esc_attr($order_id) . '">error</span>';
                        } else {
                            // Actions
                            if ($payment_status == '0' && !$trackingNumber && !$pickupNumber) {
                                // repackaging
                                echo '<span title="Reempaquetar" class="material-symbols-outlined ship-quik-repackaging ship-quik-repackaging_' . esc_attr($order_id) . '" data-key="' . esc_attr($order_id) . '"">package</span>';
                            } else {
                                if ($payment_status != '0' && !$trackingNumber && !$pickupNumber) {
                                    echo '<span title="Error pedido" class="material-symbols-outlined ship-quik-error ship-quik-error_' . esc_attr($order_id) . '" data-key="' . esc_attr($order_id) . '">error</span>';
                                } else {
                                    $labels = new GlobIterator(SHIP_QUIK_DOCSPATH . '/' . $order_id . '*' . ($fileType == 'image/png' ? '.png' : '.pdf'), GlobIterator::KEY_AS_FILENAME);

                                    if ($label_printed && $labels->count()) {
                                        // reprint label
                                        echo '<span title="Reimprimir etiqueta" class="material-symbols-outlined ship-quik-reprint-label ship-quik-print-label" data-filetype="' . esc_attr($fileType) . '" data-key="' . esc_attr($order_id) . '">update</span>';
                                    } else {
                                        // print label
                                        echo '<span title="Etiqueta pendiente" class="material-symbols-outlined ship-quik-print-label ship-quik-print-label_' . esc_attr($order_id) . '" data-filetype="' . esc_attr($fileType) . '" data-key="' . esc_attr($order_id) . '">print</span>';
                                    }
                                }
                            }
                        }

                        break;
                }
            }
        }
    }

    public function print_label()
    {
        $order_id = intval(sanitize_text_field($_REQUEST['key']));
        $fileType = urldecode(sanitize_text_field($_REQUEST['filetype']));

        $order = wc_get_order($order_id);

        if (!$order->get_meta('_shipquik_label_printed')) {
            $order->update_meta_data('_shipquik_label_printed', '1');
            $order->save();
        }

        $response = [];
        $response['labelsMimeType'] = $fileType;

        $labels = new GlobIterator(SHIP_QUIK_DOCSPATH. '/' . $order_id . '*' . ($fileType == 'image/png' ? '.png' : '.pdf'), GlobIterator::KEY_AS_FILENAME);

        foreach ($labels as $key => $label) {
            $response['labels'][] = SHIP_QUIK_DOCSURL  . '/' . $label->getFilename();
        }

        echo json_encode($response, JSON_UNESCAPED_UNICODE);

        die();
    }

    public function print_labels()
    {
        $order_id = intval(sanitize_text_field($_REQUEST['key']));
        $fileType = urldecode(sanitize_text_field($_REQUEST['filetype']));

        $order = wc_get_order($order_id);

        if (!$order->get_meta('_shipquik_label_printed')) {
            $order->update_meta_data('_shipquik_label_printed', '1');
            $order->save();
        }

        $totalPayment = $order->get_meta('_shipquik_payment');
        $supplierInfo = [];

        $shippingMethods = $order->get_shipping_methods();

        $shippingMethod = [];

        // TODO: We must have only one.
        foreach ($shippingMethods as $key => $value) {
            $shippingMethod = $value;
        }
        foreach ($shippingMethod->get_meta_data() as $meta) {
            $item = $meta->get_data();
            if ($item['key'] == '_supplier_data') {
                $supplierInfo = json_decode(str_replace('&quot;', '"', $item['value']), true, 99);

                break;
            }
        }

        $response = [];

        $response['supplier'] = $supplierInfo['supplier'];
        $response['boxes'] = $totalPayment['shipment']['boxes'];
 
        $response['labelsMimeType'] = $fileType;

        $labels = new GlobIterator(SHIP_QUIK_DOCSPATH. '/' . $order_id . '*' . ($fileType == 'image/png' ? '.png' : '.pdf'), GlobIterator::KEY_AS_FILENAME);

        foreach ($labels as $key => $label) {
            $response['labels'][] = base64_encode(file_get_contents(SHIP_QUIK_DOCSURL  . '/' . $label->getFilename()));
        }

        echo json_encode($response);

        die();
    }

    public function order_pickup()
    {
        $order_id = intval(sanitize_text_field($_REQUEST['order_id']));
        $pickup_date = sanitize_text_field($_REQUEST['pickup_date']);
        $pickup_time = sanitize_text_field($_REQUEST['pickup_time']);

        $order = wc_get_order($order_id);

        $response = [];
        $result = false;

        if ($pickup_date && $pickup_time) {
            $pickupDate = new ShipQuik_PickupDate($this->parent, $pickup_date);
            $result = $pickupDate->send();
    
            if ($result) {
                $data = [
                    'pickup_date' => $result,
                    'pickup_time' => $pickup_time,
                ];
    
                $order->update_meta_data('_shipquik_pickupdatetime', $data);
                $order->save();

                $response = ['status' => 200, 'date' => $result];

            } else {
                $response = ['status' => 200, 'error' => 'Error al procesar la consulta.'];
            }
        } else {
            $response = ['status' => 200, 'error' => 'Error al procesar la consulta.'];
        }

        echo json_encode($response, JSON_UNESCAPED_UNICODE);

        die();
    }

    public function repackaging()
    {
        $orderResponse = [];
        $orderProductsResponse = [];

        $order_id = intval(sanitize_text_field($_REQUEST['order_id']));

        $order = wc_get_order($order_id);

        $defaultPackage = ShipQuik_ShippingPackage::getDefaultPackage($this->config);

        foreach ($order->get_items() as $orderProduct) {
            $product = wc_get_product($orderProduct->get_product_id());
            $imagePath = wp_get_attachment_image_src( get_post_thumbnail_id($orderProduct->get_product_id()), 'single-post-thumbnail' );

            $width  = round($product->get_width(), 2);
            $height = round($product->get_height(), 2);
            $depth  = round($product->get_length(), 2);
            $weight = round($product->get_weight(), 2);

            if ((float) $product->get_length() == 0 || (float) $product->get_width() == 0 || (float) $product->get_height() == 0 || (float) $product->get_weight() == 0) {
                $width  = round((float) $defaultPackage['width'], 2);
                $height = round((float) $defaultPackage['height'], 2);
                $depth  = round((float) $defaultPackage['depth'], 2);
                $weight = round($defaultPackage['weight'], 2);
            }

            $orderProductsResponse[] = [
                'sku' => $product->get_sku(),
                'name' => $product->get_name(),
                'quantity' => $orderProduct['quantity'],
                'measures' => $depth . 'x' . $width . 'x' . $height,
                'image' => (isset($imagePath[0]) ? $imagePath[0] : ''),
                'remainder' => $orderProduct['quantity'],
                'measuresraw' => [
                    'width' => $width,
                    'height' => $height,
                    'depth' => $depth,
                    'weight' => $weight,
                ]
            ];
        }

        $supplierInfo = [];
        $shipmentInfo = $order->get_meta('_shipquik_payment');

        if (isset($shipmentInfo->data)) {
            $shipmentInfo = $shipmentInfo->data;
        }

        $packagesInfo = $this->config->getValueByKey('shippingPackageList');

        $shippingMethods = $order->get_shipping_methods();

        foreach ($shippingMethods as $key => $value) {
            $shippingMethod = $value;
        }

        foreach ($shippingMethod->get_meta_data() as $meta) {
            $item = $meta->get_data();
            if ($item['key'] == '_supplier_data') {
                $supplierInfo = json_decode(str_replace('&quot;', '"', $item['value']), true, 99);

                break;
            }
        }

        if ($order->get_shipping_tax()) {

        }

        $orderResponse = [
            'order_id' => $order_id,
            'products' => $orderProductsResponse,
            'packages' => $packagesInfo,
            'labels' => [
                ['label' => 'Foto', 'key' => 'image'],
                ['label' => 'Nº', 'key' => 'sku'],
                ['label' => 'Artículo', 'key' => 'name'],
                ['label' => 'Medidas', 'key' => 'measures'],
                ['label' => 'Cant.', 'key' => 'quantity'],
            ],
            'supplierInfo' => $supplierInfo,
            'shipmentInfo' => $shipmentInfo,
            'total_shipping_tax_excl' => $order->get_shipping_total(),
            'total_shipping_tax_incl' => $order->get_shipping_total(),
            'currency_symbol' => get_woocommerce_currency_symbol(),
        ];

        if ($orderResponse) {
            $orderResponse = ['status' => 200, 'order' => $orderResponse];
        } else {
            $orderResponse = ['status' => 200, 'error' => 'Error al procesar la consulta.'];
        }

        echo json_encode($orderResponse, JSON_UNESCAPED_UNICODE);

        die();
    }

    public function repackaging_save()
    {
        $order_id = intval(sanitize_text_field($_REQUEST['order_id']));
        $boxes = $_REQUEST['data'];

        $order = wc_get_order($order_id);

        $response = [];
        $result = false;

        $result = new ShipQuik_Shipments($this->parent, $order_id, $boxes);
        if (!empty($result->data)) {

            $order->update_meta_data('_shipquik_payment', $result->data);
            $order->save();

            $result->data['order_id'] = $order_id;
            $response = ['status' => 200, 'save' => $result->data];
        } else {
            $response = ['status' => 200, 'error' => 'Error al procesar la consulta.'];
        }

        echo json_encode($response, JSON_UNESCAPED_UNICODE);

        die();
    }

    public function repackaging_totals()
    {
        $order_id = intval(sanitize_text_field($_REQUEST['order_id']));
        $boxes = $_REQUEST['data'];

        $order = wc_get_order($order_id);

        $shipment = $order->get_meta('_shipquik_payment');

        $supplierInfo = new ShipQuik_Supplier($this->parent, $boxes, $shipment['shipment']['destinationAddress']);
    
        $suppliers = [
            'name' => $shipment['supplier']['supplier'],
            'services' => [$shipment['supplier']['service']],
        ];
        $supplierInfo = $supplierInfo->send($suppliers, true);
    
        $supplierInfo = (isset($supplierInfo[0]) ? $supplierInfo[0] : []);
    
        $price = 0;
        if (!empty($supplierInfo)) {
            $price = $supplierInfo['salesPrice'];
        }
    
        $result = new ShipQuik_Shipments($this->parent, $order_id, $boxes, false, $price);
    
        if (!empty($result->data)) {
            $result->data['order_id'] = $order_id;
            $response = ['status' => 200, 'totals' => $result->data];
        } else {
            $response = ['status' => 200, 'error' => 'Error al procesar la consulta.'];
        }

        echo json_encode($response, JSON_UNESCAPED_UNICODE);

        die();
    }
}
