<?php
/**
Functions for Ship-Quik
 **/

namespace ShipQuik\WooCommerce;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use \ShipQuik\Shared\ShipQuik_API;
use \ShipQuik\Shared\ShipQuik_Suppliers;
use \ShipQuik\Shared\ShipQuik_Supplier;
use \ShipQuik\Shared\ShipQuik_Configuration;
use \ShipQuik\Shared\ShipQuik_Helper;

class ShipQuik_Shipments
{
    private static $_instance = null;

    protected $parent;

    protected $order_id;

    protected $boxes;

    protected $price;

    protected $save;

    protected $config;

    public $data;

    public function __construct($parent = null, $order_id = 0, $boxes = [], $save = true, $price = 0)
    {
        $this->parent = $parent;
        $this->order_id = $order_id;
        $this->boxes = $boxes;
        $this->price = $price;
        $this->save = $save;

        $this->config = $parent->config;

        $this->data = $this->getData($this->order_id, $this->boxes, $this->save, $this->price);
    }

    private function getData($order_id, $boxes, $save, $price)
    {
        $order = wc_get_order($this->order_id);

        $supplierInfo = [];

        // HACK: leer totalPayment
        $totalPayment = $order->get_meta('_shipquik_payment');

        $customer_dni = $order->get_meta('_ship_quik_customer_dni');

        $shippingMethods = $order->get_shipping_methods();

        $shippingMethod = [];

        // TODO: We must have only one.
        foreach ($shippingMethods as $key => $value) {
            $shippingMethod = $value;
        }

        $shipment = [];

        $senderAddress = self::getSenderAddress($this->config);
        $customerAddress = [];

        if (ShipQuik_Configuration::get_option('woocommerce_ship_to_destination') == 'shipping') {
            $customerAddress = $order->get_address('shipping');
            $customerAddressBilling = $order->get_address('billing');
            $customerAddress['email'] = $customerAddressBilling['email'];
            $customerAddress['phone'] = $customerAddressBilling['phone'];
        } else {
            $customerAddress = $order->get_address('billing');
        }

        // Shipment Object
        $shipment['shipment']['isKg']  = true;

        if (empty($boxes)) {
            $shipment['shipment']['boxes'] = ShipQuik_ShippingPackage::getBoxes($this->config, $order->get_items());
        } else {
            $shipment['shipment']['boxes'] = $boxes;
        }

        $shipment['shipment']['originAddress'] = [
            'street'      => $senderAddress['Street'],
            'city'        => $senderAddress['City'],
            'countryCode' => $senderAddress['CountryCode'],
            'postalCode'  => $senderAddress['PostalCode'],
            'floor'       => '',
            'door'        => ''
        ];

        $shipment['shipment']['destinationAddress'] = [
            'street'      => $customerAddress['address_1'] . ' ' .$customerAddress['address_2'],
            'city'        => $customerAddress['city'],
            'countryCode' => $customerAddress['country'],
            'postalCode'  => $customerAddress['postcode'],
            'floor'       => "",
            'door'        => ""
        ];

        $comments = $order->get_customer_note();
        if (!$comments) {
            $comments = '';
        }

        $contact = $customerAddress['first_name'] . ' ' . $customerAddress['last_name'];

        $pickupDatetime = $order->get_meta('_shipquik_pickupdatetime');

        // Pickup object
        $shipment['pickup'] = [
            "name"      => $this->config->getValueByKey('senderAddressCompany'),
            "cif"       => $this->config->getValueByKey('senderAddressCif'),
            "phone"     => [
                "countryCode" => (string) $senderAddress['CountryCode'],
                "phoneNumber" => $this->config->getValueByKey('senderAddressPhoneNumber')
            ],
            "email"     => $this->config->getValueByKey('senderAddressEmail'),
            "comments"  => $this->config->getValueByKey('comments'),
            "attention" => '',
            'date' => (isset($pickupDatetime['pickup_date']) ? $pickupDatetime['pickup_date'] : ''),
            'schedule' => (isset($pickupDatetime['pickup_time']) ? $pickupDatetime['pickup_time'] : '1')
        ];

        $name = ($customerAddress['company'] ? $customerAddress['company'] : $customerAddress['first_name'] . ' ' . $customerAddress['last_name']);

        $cif = $order->get_meta($this->config->getValueByKey('customerFieldDNI'));

        // Delivery object
        $shipment['delivery'] = [
            "name"      => $name,
            "cif"       => $cif ? $cif : $order->get_meta('_ship_quik_customer_dni'),
            "phone"     => [
                "countryCode" => (string) $customerAddress['country'],
                "phoneNumber" => $customerAddress['phone'],
            ],
            "email"     => $customerAddress['email'],
            "comments"  => $comments,
            "attention" => $contact,
        ];

        $supplierInfo = [];

        foreach ($shippingMethod->get_meta_data() as $meta) {
            $item = $meta->get_data();
            if ($item['key'] == '_supplier_data') {
                $supplierInfo = json_decode(str_replace('&quot;', '"', $item['value']), true, 99);

                break;
            }
        }

        if ($price > 0) {
            $supplierInfo['salesPrice'] = $price;
        }

        // Supplier object
        $shipment["supplier"] = $supplierInfo;
        $shipment["supplier"]['currency'] = "";
        $shipment["supplier"]['serviceCode'] = (isset($supplierInfo['serviceCode']) ? $supplierInfo['serviceCode'] : '');

        $shipment['dua'] = [
            'duaSale' => $supplierInfo['dua'],
        ];

        $taricLines = [];
        if ($supplierInfo['isTaric']) {
            foreach ($shipment['shipment']['boxes'] as $_box) {
                $taricLines[] = [
                    'quantity' => 1,
                    'description' => 'Cajas, bolsas de papel | 358',
                    'value' => 10,
                ];
            }
        }
        $shipment['insurance'] = [
            'insurancePrice' => 0,
            'declaredValue' => 0,
            'insuranceCost' => 0,
            'declaredObjects' => 'Otros',
            'contentAgreement' => true,
            'isInternational' => $supplierInfo['isTaric'],
            'isTaric' => $supplierInfo['isTaric'],
            'tarics' => $taricLines,
        ];

        $shipment['iva'] = [
            'ivaPercentage' => 0,
            'ivaVentaName' => 'NOSUJETO',
            'ivaCompraName' => 'IGIAND45',
            'operationVName' => 'VNAC',
            'operationCName' => 'SUJPAS',
        ];

        $shipment['payment'] = [
            "total"  => round($supplierInfo['salesPrice'] + $supplierInfo['dua'], 2),
            "sale"   => round($supplierInfo['salesPrice'], 2),
            "method" => (string) $this->config->getPaymentMethod()
        ];

        $uuid = ShipQuik_Helper::getUUID($supplierInfo['supplier']);

        $shipment['seriesInfo'] = [
            'salesOrder' => 'PV-WC',
            'purchaseOrder' => 'PC-WC',
            'invoice' => 'FV-WC',
        ];

        $shipment['additionalProperties'] = [
            'sendEmails' => false,
        ];

        if (isset($totalPayment['reference'])) {
            $shipment['orderId'] = $totalPayment['reference'];
            $shipment['reference'] = $totalPayment['reference'];
        } else {
            $shipment['orderId'] = $uuid;
            $shipment['reference'] = $uuid;
        }

        $shipment["userEmail"] = $this->config->getValueByKey('senderAddressEmail');
        $shipment["clientId"] = $this->config->getIdCliente();

        if ($this->save) {
            $order->update_meta_data('_shipquik_payment', $shipment);
        }

        $order->save();

        return $shipment;
    }

    public static function getSenderAddress($config)
    {
        $senderAddress = [];

        $senderAddress['Street'] = $config->getValueByKey('senderAddressStreet');
        $senderAddress['City'] = $config->getValueByKey('senderAddressCity');
        $senderAddress['CountryCode'] = $config->getValueByKey('senderAddressCountryCode');
        $senderAddress['PostalCode'] = $config->getValueByKey('senderAddressPostalCode');

        $senderAddress['Country'] = WC()->countries->countries[$config->getValueByKey('senderAddressCountryCode')];

        return $senderAddress;
    }

    public function send()
    {
        $params = [];
        $params['Baseurl']  = $this->config->getValueByKey('endpoints')['createShipment'];
        $params['Data']     = $this->data;

        $curl = new ShipQuik_API($this->parent, $params, true);
        $result = $curl->runCall();

        if ($result == "renew_token") {
            $this->config->reNewAccessToken();

            $result = $curl->runCall();
        }

        return $result;
    }

    public static function instance()
    {
        if (! isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }
        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
